@@ -1,4 +1,4 @@
-"""Provide an asychronous equivalent to *input*."""
+"""Provide an asynchronous equivalent to *input*."""
 
 import os
 import sys
@@ -6,6 +6,8 @@ import stat
 import weakref
 import asyncio
 from collections import deque
+from threading import Thread
+from concurrent.futures import Future
 
 from . import compat
 
@@ -26,6 +28,34 @@ def is_pipe_transport_compatible(pipe):
     return True
 
 
+async def run_as_daemon(func, *args):
+    future = Future()
+    future.set_running_or_notify_cancel()
+
+    # A bug in python 3.7 makes it a bad idea to set a BaseException
+    # in a wrapped future (see except statement in asyncio.Task.__wakeup)
+    # Instead, we'll wrap base exceptions into exceptions and unwrap them
+    # on the other side of the call.
+    class BaseExceptionWrapper(Exception):
+        pass
+
+    def daemon():
+        try:
+            result = func(*args)
+        except Exception as e:
+            future.set_exception(e)
+        except BaseException as e:
+            future.set_exception(BaseExceptionWrapper(e))
+        else:
+            future.set_result(result)
+
+    Thread(target=daemon, daemon=True).start()
+    try:
+        return await asyncio.wrap_future(future)
+    except BaseExceptionWrapper as exc:
+        raise exc.args[0]
+
+
 def protect_standard_streams(stream):
     if stream._transport is None:
         return
@@ -98,14 +128,14 @@ class NonFileStreamReader:
         return self.eof
 
     async def readline(self):
-        data = await self.loop.run_in_executor(None, self.stream.readline)
+        data = await run_as_daemon(self.stream.readline)
         if isinstance(data, str):
             data = data.encode()
         self.eof = not data
         return data
 
     async def read(self, n=-1):
-        data = await self.loop.run_in_executor(None, self.stream.read, n)
+        data = await run_as_daemon(self.stream.read, n)
         if isinstance(data, str):
             data = data.encode()
         self.eof = not data
